package net.grandnature.android.soundball;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.graphics.drawable.Drawable;

/**
 * Soundballアプリケーションの画面オブジェクトの基底クラス。
 */
public abstract class SoundballCharacter {
	protected int width, height; // 横幅と高さ
	protected Rect viewRect, rect; // 描画領域全体の矩形と、自オブジェクトの矩形
	protected Drawable drawable; // 絵
	protected Context context;
	private Paint paint; // 線

	public SoundballCharacter(Context context, Drawable drawable, Rect viewRect) {
		this.context = context;
		this.drawable = drawable;
		// Drawableの横幅と高さを矩形のサイズとして利用する
		this.width = drawable.getIntrinsicWidth();
		this.height = drawable.getIntrinsicHeight();
		this.rect = new Rect(0, 0, width, height);
		// 描画領域全体の矩形
		this.viewRect = viewRect;
		// ペンの初期化
		this.paint = new Paint(Paint.ANTI_ALIAS_FLAG);
		this.paint.setColor(Color.BLACK);
		this.paint.setStrokeWidth(3);
		this.paint.setStyle(Paint.Style.STROKE);
		init();
	}

	// 初期化
	protected void init() {};

	/**
	 * 描画する
	 * @param canvas
	 */
	public void draw(Canvas canvas) {
		drawable.setBounds(rect);
		drawable.draw(canvas);
	}

	/**
	 * 自オブジェクトとターゲットの間に線を引く
	 * @param canvas
	 * @param target ターゲット
	 */
	public void drawLine(Canvas canvas, SoundballCharacter target) {
		canvas.drawLine(rect.centerX(), rect.exactCenterY(), target.rect
				.centerX(), target.rect.centerY(), paint);
	}

	/**
	 * 自オブジェクトの中に座標（x, y）が含まれるかを確認する
	 * @param x
	 * @param y
	 * @return 含まれる:true/含まれない:false
	 */
	public boolean isInside(float x, float y) {
		return rect.contains((int) x, (int) y);
	}

	/**
	 * 自オブジェクトとターゲットの領域が重なっているかを確認する
	 * @param target ターゲット
	 * @return 重なっている:true/重なっていない:false
	 */
	public boolean doesIntersect(SoundballCharacter target) {
		return rect.intersect(target.rect);
	}

	/**
	 * ターゲットとの距離を測る
	 * @param target 対象のオブジェクト
	 * @return 距離
	 */
	public int getDistance(SoundballCharacter target) {
		int w = rect.centerX() - target.rect.centerX();
		int h = rect.centerY() - target.rect.centerY();
		return (int) Math.sqrt(Math.pow(w, 2) + Math.pow(h, 2));
	}

	/**
	 * dx,dyだけ移動する
	 * @param dx
	 * @param dy
	 */
	public void offset(int dx, int dy) {
		rect.offset(dx, dy);
		adjustViewBounds();
	}

	/**
	 * x,yに移動する
	 * @param x
	 * @param y
	 */
	public void set(int x, int y) {
		int w = width / 2, h = height / 2;
		rect.set(x - w, y - h, x + w, y + h);
		adjustViewBounds();
	}
	// 描画領域内に収まっていなければ調整する
	private void adjustViewBounds() {
		if (!viewRect.contains(rect)) {
			int dx = 0, dy = 0;
			if (rect.left < viewRect.left) 
				dx = viewRect.left - rect.left;
			if (rect.top < viewRect.top) 
				dy = viewRect.top - rect.top;
			if (viewRect.right < rect.right) 
				dx = viewRect.right - rect.right;
			if (viewRect.bottom < rect.bottom) 
				dy = viewRect.bottom - rect.bottom;
			offset(dx, dy);
		}
	}

	/**
	 * ペンの色を変える
	 * @param color 色
	 */
	public void setPenColor(int color) {
		Paint tmp = new Paint(Paint.ANTI_ALIAS_FLAG);
		tmp.setColor(color);
		tmp.setStrokeWidth(paint.getStrokeWidth());
		tmp.setStyle(Paint.Style.STROKE);
		paint = tmp;
	}

	/**
	 * ペンの太さを変える
	 * @param width 太さ
	 */
	public void setPenStrokeWidth(int width) {
		Paint tmp = new Paint(Paint.ANTI_ALIAS_FLAG);
		tmp.setColor(paint.getColor());
		tmp.setStrokeWidth(width);
		tmp.setStyle(Paint.Style.STROKE);
		paint = tmp;
	}
}